import { InlineTable } from '@cubejs-backend/base-driver';
import { CubeStoreDriver } from '@cubejs-backend/cubestore-driver';
import { PreAggTableToTempTable, Query, QueryBody, QueryCache, QueryWithParams } from './QueryCache';
import { DriverFactory, DriverFactoryByDataSource } from './DriverFactory';
import { QueryQueue } from './QueryQueue';
import { CacheAndQueryDriverType } from './QueryOrchestrator';
import { PreAggregationLoadCache } from './PreAggregationLoadCache';
export declare const LAMBDA_TABLE_PREFIX = "lambda";
export declare function version(cacheKey: any): string;
export declare function getLastUpdatedAtTimestamp(timestamps: (number | undefined)[]): number | undefined;
export declare function getStructureVersion(preAggregation: any): string;
export type VersionEntry = {
    'table_name': string;
    'content_version': string;
    'structure_version': string;
    'last_updated_at': number;
    'build_range_end'?: string;
    'naming_version'?: number;
};
export type VersionEntriesObj = {
    versionEntries: VersionEntry[];
    byStructure: {
        [key: string]: VersionEntry;
    };
    byContent: {
        [key: string]: VersionEntry;
    };
    byTableName: {
        [key: string]: VersionEntry;
    };
};
export type InvalidationKeys = unknown[];
export type TableCacheEntry = {
    table_name?: string;
    TABLE_NAME?: string;
    build_range_end?: string;
};
export type QueryDateRange = [string, string];
export type PartitionRanges = {
    buildRange: QueryDateRange;
    partitionRanges: QueryDateRange[];
};
type IndexDescription = {
    sql: QueryWithParams;
    indexName: string;
};
export type LoadPreAggregationResult = {
    targetTableName: string;
    refreshKeyValues: any[];
    lastUpdatedAt: number;
    buildRangeEnd: string;
    lambdaTable?: InlineTable;
    queryKey?: any[];
    rollupLambdaId?: string;
    partitionRange?: QueryDateRange;
};
export type PreAggregationTableToTempTable = [string, LoadPreAggregationResult];
export type LambdaOptions = {
    maxSourceRows: number;
};
export type LambdaQuery = {
    sqlAndParams: QueryWithParams;
    cacheKeyQueries: any[];
};
export type PreAggregationDescription = {
    preAggregationsSchema: string;
    type: 'rollup' | 'originalSql';
    preAggregationId: string;
    priority: number;
    dataSource: string;
    external: boolean;
    previewSql: QueryWithParams;
    timezone: string;
    indexesSql: IndexDescription[];
    invalidateKeyQueries: QueryWithParams[];
    partitionInvalidateKeyQueries: QueryWithParams[];
    structureVersionLoadSql: QueryWithParams;
    sql: QueryWithParams;
    loadSql: QueryWithParams;
    tableName: string;
    matchedTimeDimensionDateRange: QueryDateRange;
    granularity: string;
    partitionGranularity: string;
    preAggregationStartEndQueries: [QueryWithParams, QueryWithParams];
    timestampFormat: string;
    timestampPrecision: number;
    expandedPartition: boolean;
    unionWithSourceData: LambdaOptions;
    buildRangeStart?: string;
    buildRangeEnd?: string;
    updateWindowSeconds?: number;
    sealAt?: string;
    rollupLambdaId?: string;
    lastRollupLambda?: boolean;
};
export declare const tablesToVersionEntries: (schema: any, tables: TableCacheEntry[]) => VersionEntry[];
type PreAggregationsOptions = {
    maxPartitions: number;
    maxSourceRowLimit: number;
    preAggregationsSchemaCacheExpire?: number;
    loadCacheQueueOptions?: any;
    queueOptions?: (dataSource: string) => Promise<{
        concurrency: number;
        continueWaitTimeout?: number;
        executionTimeout?: number;
        orphanedTimeout?: number;
        heartBeatInterval?: number;
    }>;
    cubeStoreDriverFactory?: () => Promise<CubeStoreDriver>;
    continueWaitTimeout?: number;
    cacheAndQueueDriver?: CacheAndQueryDriverType;
    skipExternalCacheAndQueue?: boolean;
};
type PreAggregationQueryBody = QueryBody & {
    preAggregationsLoadCacheByDataSource?: {
        [key: string]: PreAggregationLoadCache;
    };
};
export declare class PreAggregations {
    private readonly redisPrefix;
    private readonly driverFactory;
    private readonly logger;
    private readonly queryCache;
    options: PreAggregationsOptions;
    externalDriverFactory: DriverFactory;
    structureVersionPersistTime: any;
    private readonly touchTablePersistTime;
    readonly dropPreAggregationsWithoutTouch: boolean;
    private readonly usedTablePersistTime;
    private readonly externalRefresh;
    private readonly loadCacheQueue;
    private readonly queue;
    private readonly getQueueEventsBus;
    private readonly touchCache;
    constructor(redisPrefix: string, driverFactory: DriverFactoryByDataSource, logger: any, queryCache: QueryCache, options: any);
    protected tablesUsedRedisKey(tableName: string): string;
    protected tablesTouchRedisKey(tableName: string): string;
    protected refreshEndReachedKey(): string;
    addTableUsed(tableName: string): Promise<void>;
    tablesUsed(): Promise<any[]>;
    updateLastTouch(tableName: string): Promise<void>;
    tablesTouched(): Promise<any[]>;
    updateRefreshEndReached(): Promise<{
        key: string;
        bytes: number;
    }>;
    getRefreshEndReached(): Promise<number>;
    /**
     * Determines whether the partition table is already exists or not.
     */
    isPartitionExist(request: string, external: boolean, dataSource: string, schema: string, table: string, key: any, token: string): Promise<[boolean, string]>;
    loadAllPreAggregationsIfNeeded(queryBody: PreAggregationQueryBody): Promise<{
        preAggregationsTablesToTempTables: PreAggTableToTempTable[];
        values: null | string[];
    }>;
    /**
     * Determines whether range queries for the preAggregations from the
     * queryBody were cached or not.
     */
    checkPartitionsBuildRangeCache(queryBody: any): Promise<any[]>;
    expandPartitionsInPreAggregations(queryBody: Query): Promise<Query>;
    getQueue(dataSource?: string): Promise<QueryQueue>;
    /**
     * Returns registered queries queues hash table.
     */
    getQueues(): {
        [dataSource: string]: QueryQueue;
    };
    getLoadCacheQueue(dataSource?: string): QueryQueue;
    static preAggregationQueryCacheKey(preAggregation: any): any;
    static targetTableName(versionEntry: any): string;
    static noPreAggregationPartitionsBuiltMessage(preAggregations: PreAggregationDescription[]): string;
    static structureVersion(preAggregation: any): string;
    getVersionEntries(preAggregations: PreAggregationDescription[], requestId: any): Promise<VersionEntry[][]>;
    getQueueState(dataSource: string): Promise<any>;
    cancelQueriesFromQueue(queryKeys: string[], dataSource: string): Promise<void[]>;
}
export {};
//# sourceMappingURL=PreAggregations.d.ts.map